package com.ozs.web.controller.plan;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageInfo;
import com.ozs.common.annotation.Log;
import com.ozs.common.constant.Constants;
import com.ozs.common.constant.ModularConstans;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysRole;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.enums.DataIsDelete;
import com.ozs.common.exception.ServiceException;
import com.ozs.common.utils.PageUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.framework.web.service.TokenService;
import com.ozs.plan.doman.PlanQuarter;
import com.ozs.plan.doman.ProvisionalPlan;
import com.ozs.plan.doman.vo.requestVo.ProvisionalPlanVo;
import com.ozs.plan.doman.vo.responseVo.PlanQuarterResponseVo;
import com.ozs.plan.service.PlanQuarterService;
import com.ozs.plan.service.ProvisionalPlanService;
import com.ozs.plan.service.impl.PlanQuarterServiceImpl;
import com.ozs.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 临时计划
 *
 * @author sunhh
 */
@Api(tags = "临时计划")
@RestController
@RequestMapping("/plan/provisionalPlan")
public class ProvisionalPlanController extends BaseController {

    @Autowired
    private PlanQuarterService quarterService;
    @Autowired
    private PlanQuarterServiceImpl quarterServiceImpl;
    @Autowired
    private ProvisionalPlanService provisionalPlanService;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private ISysDeptService iSysDeptService;
    @Autowired
    private PlanQuarterServiceImpl planQuarterServiceImpl;

    @ApiOperation(value = "新增临时计划")
    @PostMapping("/insertProvisionalPlan")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:insertProvisionalPlan')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.INSERT)
    public AjaxResult insertProvisionalPlan(@RequestBody ProvisionalPlan provisionalPlan, HttpServletRequest request) {
        //获取采购单位
        LoginUser loginUser = tokenService.getLoginUser(request);
        provisionalPlan.setPurchaseDeptId(String.valueOf(loginUser.getDeptId()));
        provisionalPlan.setCreated(String.valueOf(loginUser.getUserId()));
        provisionalPlan.setCreateTime(new Date());
        // planType计划类型（0：季度计划，1：临时计划）
        provisionalPlan.setPlanType("1");
        // 项目状态(0:计划待提交，1:计划待审核，2:计划已退回，3:计划已审核)
        provisionalPlan.setProjectStatus("0");
        provisionalPlan.setIsExcess(planQuarterServiceImpl.isExcessOrNo(provisionalPlan.getProjectType(), provisionalPlan.getEvaluation()));
        return toAjax(provisionalPlanService.save(provisionalPlan));
    }

    @ApiOperation(value = "删除临时计划", notes = "必传 plan_practical_id")
    @PostMapping("/deleteProvisionalPlan")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:deleteProvisionalPlan')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.DELETE)
    public AjaxResult deleteProvisionalPlan(@RequestBody ProvisionalPlan provisionalPlan) {
        if (StringUtils.isNull(provisionalPlan) || StringUtils.isNull(provisionalPlan.getPlanPracticalId())) {
            return error("临时计划id不能为空");
        }
        return toAjax(provisionalPlanService.deleteProvisionalPlanById(provisionalPlan.getPlanPracticalId()));
    }

    @ApiOperation(value = "修改临时计划", notes = "必传 plan_year_id 及修改数据")
    @PostMapping("/updateProvisionalPlan")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:updateProvisionalPlan')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.UPDATE)
    public AjaxResult updateProvisionalPlan(@RequestBody ProvisionalPlanVo provisionalPlan) {
        if (StringUtils.isNull(provisionalPlan) || StringUtils.isNull(provisionalPlan.getPlanPracticalId())) {
            return error("临时计划id和修改数据不能为空");
        }
        return toAjax(provisionalPlanService.updateProvisionalPlanById(provisionalPlan));
    }

    @ApiOperation(value = "查询临时计划")
    @PostMapping("/selectProvisionalPlan")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:selectProvisionalPlan')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.QUERY)
    public AjaxResult selectProvisionalPlan(@RequestBody ProvisionalPlanVo vo, HttpServletRequest request) {
        if (StringUtils.isNull(vo)
                || StringUtils.isNull(vo.getPageNum())
                || StringUtils.isNull(vo.getPageSize())) {
            throw new ServiceException("分页参数不能为空！");
        }
        List<PlanQuarterResponseVo> planQuarterList = new ArrayList<>();
        try {
            LambdaQueryWrapper<PlanQuarter> lw = new LambdaQueryWrapper<>();
            lw.eq(PlanQuarter::getDelFlay, DataIsDelete.DataNOTDelete.getCode());
            lw.eq(PlanQuarter::getPlanType, "1");
            if (!ObjectUtils.isEmpty(vo.getProjectName())) {
                lw.like(PlanQuarter::getProjectName, vo.getProjectName());
            }
            if (!ObjectUtils.isEmpty(vo.getPurchaseServices())) {
                lw.eq(PlanQuarter::getPurchaseServices, vo.getPurchaseServices());
            }
            if (!ObjectUtils.isEmpty(vo.getIsExcess())) {
                lw.eq(PlanQuarter::getIsExcess, vo.getIsExcess());
            }
            if (!ObjectUtils.isEmpty(vo.getProjectStatus())) {
                lw.eq(PlanQuarter::getProjectStatus, vo.getProjectStatus());
            }
            if (!ObjectUtils.isEmpty(vo.getStartTime())) {
                lw.ge(PlanQuarter::getPlanDemandSubTime, vo.getStartTime());
            }
            if (!ObjectUtils.isEmpty(vo.getEndTime())) {
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
                SimpleDateFormat dateFormatT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
                Date parse = dateFormatT.parse(dateFormat.format(vo.getEndTime()) + " 23:59:59:999");
                lw.le(PlanQuarter::getPlanDemandSubTime, parse);
            }

            LoginUser loginUser = tokenService.getLoginUser(request);
            // 添加数据权限
            List<String> roleKeys = loginUser.getUser().getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.toList());
            if (roleKeys.contains(Constants.DEMAND_UNIT)) {
                // 需求单位
                /*（purchase_dept_id = 当前用户deptID） */
                lw.eq(PlanQuarter::getPurchaseDeptId, loginUser.getDeptId());
            } else if (roleKeys.contains(Constants.PURCHASING_MANAGEMENT)
                    || roleKeys.contains(Constants.PURCHASE_SERVICES)) {
                // 采购管理部门  或  采购办
                SysDept sysDept = new SysDept();
                sysDept.setParentId(loginUser.getDeptId());
                sysDept.setStatus("0");
                List<Long> childDeptIds = iSysDeptService.selectDeptList(sysDept)
                        .stream()
                        .map(SysDept::getDeptId)
                        .collect(Collectors.toList());
                if (ObjectUtils.isEmpty(childDeptIds)) {
                    lw.and((wrapper) -> {
                        wrapper.eq(PlanQuarter::getIsExcess, 0);
                        wrapper.eq(PlanQuarter::getPurchaseDeptId, loginUser.getDeptId());
                    });
                } else {
                    lw.and((wrapper) -> {
                        wrapper.eq(PlanQuarter::getIsExcess, 0);
                        wrapper.eq(PlanQuarter::getPurchaseDeptId, loginUser.getDeptId());
                    })
                            .or((wrapper) -> {
                                wrapper.eq(PlanQuarter::getIsExcess, 1);
                                wrapper.in(PlanQuarter::getPurchaseDeptId, childDeptIds);
                            });
                }
            }
            lw.orderBy(true, false, PlanQuarter::getCreateTime);
            List<PlanQuarter> list = quarterService.list(lw);
            planQuarterList = quarterServiceImpl.changeTo(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Page pages = PageUtils.getPages(vo.getPageNum().intValue(), vo.getPageSize().intValue(), planQuarterList);
        PageInfo<ProvisionalPlan> provisionalPlanPageInfo = new PageInfo<>();
        if (!ObjectUtils.isEmpty(pages) && pages.getRecords().size() > 0) {
            provisionalPlanPageInfo.setList(pages.getRecords());
            provisionalPlanPageInfo.setTotal(pages.getTotal());
            provisionalPlanPageInfo.setPages(Integer.parseInt(pages.getPages() + ""));
        }
        return success(provisionalPlanPageInfo);
    }

    @ApiOperation(value = "查询临时计划-审核单位")
    @PostMapping("/selectProvisionalPlanAudit")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:selectProvisionalPlanAudit')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.QUERY)
    public AjaxResult selectProvisionalPlanAudit(@RequestBody ProvisionalPlanVo vo, HttpServletRequest request) {
        if (StringUtils.isNull(vo)
                || StringUtils.isNull(vo.getPageNum())
                || StringUtils.isNull(vo.getPageSize())) {
            throw new ServiceException("分页参数不能为空！");
        }
        List<PlanQuarterResponseVo> planQuarterList = new ArrayList<>();
        try {
            LambdaQueryWrapper<PlanQuarter> lw = new LambdaQueryWrapper<>();
            lw.eq(PlanQuarter::getDelFlay, DataIsDelete.DataNOTDelete.getCode());
            lw.eq(PlanQuarter::getPlanType, "1");
            if (!ObjectUtils.isEmpty(vo.getProjectName())) {
                lw.like(PlanQuarter::getProjectName, vo.getProjectName());
            }
            if (!ObjectUtils.isEmpty(vo.getPurchaseServices())) {
                lw.eq(PlanQuarter::getPurchaseServices, vo.getPurchaseServices());
            }
            if (!ObjectUtils.isEmpty(vo.getIsExcess())) {
                lw.eq(PlanQuarter::getIsExcess, vo.getIsExcess());
            }
            if (!ObjectUtils.isEmpty(vo.getProjectStatus())) {
                lw.eq(PlanQuarter::getProjectStatus, vo.getProjectStatus());
            } else {
                lw.apply("  project_status IN (1,3)", "");
            }
            if (!ObjectUtils.isEmpty(vo.getStartTime())) {
                lw.ge(PlanQuarter::getPlanDemandSubTime, vo.getStartTime());
            }
            if (!ObjectUtils.isEmpty(vo.getEndTime())) {
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
                SimpleDateFormat dateFormatT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
                Date parse = dateFormatT.parse(dateFormat.format(vo.getEndTime()) + " 23:59:59:999");
                lw.le(PlanQuarter::getPlanDemandSubTime, parse);
            }

            LoginUser loginUser = tokenService.getLoginUser(request);
            // 添加数据权限
            List<String> roleKeys = loginUser.getUser().getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.toList());
            if (roleKeys.contains(Constants.DEMAND_UNIT)) {
                // 需求单位
                /*（purchase_dept_id = 当前用户deptID） */
                lw.eq(PlanQuarter::getPurchaseDeptId, loginUser.getDeptId());
            } else if (roleKeys.contains(Constants.PURCHASING_MANAGEMENT)
                    || roleKeys.contains(Constants.PURCHASE_SERVICES)) {
                // 采购管理部门  或  采购办
                SysDept sysDept = new SysDept();
                sysDept.setParentId(loginUser.getDeptId());
                sysDept.setStatus("0");
                List<Long> childDeptIds = iSysDeptService.selectDeptList(sysDept)
                        .stream()
                        .map(SysDept::getDeptId)
                        .collect(Collectors.toList());
                if (ObjectUtils.isEmpty(childDeptIds)) {
                    lw.and((wrapper) -> {
                        wrapper.eq(PlanQuarter::getIsExcess, 0);
                        wrapper.eq(PlanQuarter::getPurchaseDeptId, loginUser.getDeptId());
                    });
                } else {
                    lw.and((wrapper) -> {
                        wrapper.eq(PlanQuarter::getIsExcess, 0);
                        wrapper.eq(PlanQuarter::getPurchaseDeptId, loginUser.getDeptId());
                    })
                            .or((wrapper) -> {
                                wrapper.eq(PlanQuarter::getIsExcess, 1);
                                wrapper.in(PlanQuarter::getPurchaseDeptId, childDeptIds);
                            });
                }
            }
            lw.orderBy(true, false, PlanQuarter::getCreateTime);
            List<PlanQuarter> list = quarterService.list(lw);
            planQuarterList = quarterServiceImpl.changeTo(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Page pages = PageUtils.getPages(vo.getPageNum().intValue(), vo.getPageSize().intValue(), planQuarterList);
        PageInfo<ProvisionalPlan> provisionalPlanPageInfo = new PageInfo<>();
        if (!ObjectUtils.isEmpty(pages) && pages.getRecords().size() > 0) {
            provisionalPlanPageInfo.setList(pages.getRecords());
            provisionalPlanPageInfo.setSize(Math.toIntExact(pages.getSize()));
            provisionalPlanPageInfo.setPageNum(Math.toIntExact(pages.getCurrent()));
            provisionalPlanPageInfo.setTotal(pages.getTotal());
            provisionalPlanPageInfo.setPages(Integer.parseInt(pages.getPages() + ""));
        }
        return AjaxResult.success(provisionalPlanPageInfo);
    }

    @ApiOperation(value = "查询临时计划详情")
    @PostMapping("/selectById")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:selectById')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.QUERY)
    public AjaxResult selectById(@RequestBody ProvisionalPlanVo provisionalPlanVo) {
        if (StringUtils.isNull(provisionalPlanVo) || StringUtils.isNull(provisionalPlanVo.getPlanPracticalId())) {
            throw new ServiceException("id不能为空！");
        }
        return provisionalPlanService.seletById(provisionalPlanVo.getPlanPracticalId());
    }

    @ApiOperation(value = "导入临时计划")
    @PostMapping("/importProvisionalPlan")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:importProvisionalPlan')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.INSERT)
    public AjaxResult importProvisionalPlan(MultipartFile file, boolean updateSupport) throws Exception {
        ExcelUtil<ProvisionalPlan> util = new ExcelUtil<>(ProvisionalPlan.class);
        List<ProvisionalPlan> provisionalPlans = util.importExcel(file.getInputStream());
        if (StringUtils.isNull(provisionalPlans) || provisionalPlans.size() == 0) {
            throw new ServiceException("导入临时计划数据不能为空！");
        }
        //获取采购单位-
        LoginUser loginUser = getLoginUser();
        String message = provisionalPlanService.importProvisionalPlan(provisionalPlans, updateSupport, loginUser);
        return success(message);
    }

    @ApiOperation(value = "导出临时计划")
    @PostMapping("/exportProvisionalPlan")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:exportProvisionalPlan')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.EXPORT)
    public void exportProvisionalPlan(HttpServletResponse response, @RequestBody ProvisionalPlanVo provisionalPlanVo) throws Exception {
        List<ProvisionalPlan> list = provisionalPlanService.selectProvisionalPlanExport(provisionalPlanVo);
        ExcelUtil<ProvisionalPlan> util = new ExcelUtil<>(ProvisionalPlan.class);
        util.exportExcel(response, list, "临时计划数据");
    }

    @ApiOperation(value = "提交临时计划")
    @PostMapping("/commitProvisionalPlan")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:commitProvisionalPlan')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.UPDATE)
    public AjaxResult commitProvisionalPlan(@RequestBody ProvisionalPlanVo provisionalPlanVo) {
        if (StringUtils.isNull(provisionalPlanVo) || StringUtils.isNull(provisionalPlanVo.getPlanPracticalId())) {
            return error("提交的临时计划ID不能为空！");
        }
        return toAjax(provisionalPlanService.commitProvisionalPlan(provisionalPlanVo));
    }

    @ApiOperation(value = "审核通过")
    @PostMapping("/auditPass")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:auditPass')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.UPDATE)
    public AjaxResult auditPass(@RequestBody ProvisionalPlanVo provisionalPlanVo, HttpServletRequest request) {
        if (StringUtils.isNull(provisionalPlanVo) || StringUtils.isNull(provisionalPlanVo.getPlanPracticalId())) {
            return error("审核数据ID不能为空！");
        }
        LoginUser loginUser = tokenService.getLoginUser(request);
        provisionalPlanVo.setUpdated(String.valueOf(loginUser.getUserId()));
        return provisionalPlanService.auditPass(provisionalPlanVo);
    }

    @ApiOperation(value = "审核不通过")
    @PostMapping("/auditNoPass")
    @PreAuthorize("@ss.hasPermi('plan:provisionalPlan:auditNoPass')")
    @Log(title = ModularConstans.provisionalPlan, businessType = BusinessType.UPDATE)
    public AjaxResult auditNoPass(@RequestBody ProvisionalPlanVo provisionalPlanVo, HttpServletRequest request) {
        if (StringUtils.isNull(provisionalPlanVo) || StringUtils.isNull(provisionalPlanVo.getPlanPracticalId())) {
            return error("审核数据ID不能为空！");
        }
        LoginUser loginUser = tokenService.getLoginUser(request);
        provisionalPlanVo.setUpdated(String.valueOf(loginUser.getUserId()));
        return provisionalPlanService.auditNoPass(provisionalPlanVo);
    }
}
